# Generated by Django 4.0.4 on 2022-05-19 06:10

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)

import audoma.django.db.fields


class Migration(migrations.Migration):

    dependencies = [
        ("audoma_api", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="ExampleSimpleModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", audoma.django.db.fields.CharField(max_length=64)),
                ("value", audoma.django.db.fields.IntegerField(default=0, null=True)),
            ],
        ),
        migrations.AlterField(
            model_name="examplemodel",
            name="json",
            field=audoma.django.db.fields.JSONField(),
        ),
        migrations.AlterField(
            model_name="examplemodel",
            name="money_currency",
            field=audoma.django.db.fields.CurrencyField(
                choices=[
                    ("AUD", "Australian Dollar"),
                    ("GBP", "British Pound"),
                    ("CNY", "Chinese Yuan"),
                    ("EUR", "Euro"),
                    ("INR", "Indian Rupee"),
                    ("JPY", "Japanese Yen"),
                    ("NZD", "New Zealand Dollar"),
                    ("CHF", "Swiss Franc"),
                    ("USD", "US Dollar"),
                ],
                default="XYZ",
                max_length=3,
            ),
        ),
        migrations.AlterField(
            model_name="exampleperson",
            name="savings_currency",
            field=audoma.django.db.fields.CurrencyField(
                choices=[
                    ("AUD", "Australian Dollar"),
                    ("GBP", "British Pound"),
                    ("CNY", "Chinese Yuan"),
                    ("EUR", "Euro"),
                    ("INR", "Indian Rupee"),
                    ("JPY", "Japanese Yen"),
                    ("NZD", "New Zealand Dollar"),
                    ("CHF", "Swiss Franc"),
                    ("USD", "US Dollar"),
                ],
                default="PLN",
                max_length=3,
            ),
        ),
        migrations.CreateModel(
            name="ExampleTagModel",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", audoma.django.db.fields.CharField(max_length=32)),
                (
                    "item",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="tags",
                        to="audoma_api.examplesimplemodel",
                    ),
                ),
            ],
        ),
    ]
